////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	antivirus.h
// Desc:	Central header, included by all game components
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_ANTIVIRUS_H
#define HDD_ANTIVIRUS_CORE_ANTIVIRUS_H


//-----------------------------------------------------------------------------
// Include directives

// windows
#include <windows.h>
#include <windowsx.h>
#include <direct.h>
#include <commdlg.h>
#include <commctrl.h>
#include <Shlobj.h>
#include <Shlwapi.h>
#include <time.h>

// crt
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <io.h>

// direct x

#ifndef DIRECTINPUT_VERSION
#define DIRECTINPUT_VERSION 0x0800
#endif

#include <D3D8.h>
#include <D3Dx8.h>
#include <dinput.h>

#include "resource.h"

// antivirus hdrs
#include "avBasetypes.h"
#include "avGameDefs.h"
#include "c3DRender.h"
#include "c2DRender.h"
#include "c3DGameView.h"
#include "cMousePtr.h"
#include "cFrameLimiter.h"
#include "cViralData.h"
#include "cViralChain.h"
#include "cCureData.h"
#include "cCureSeq.h"
#include "cCureChain.h"
#include "cGameplay.h"
#include "cHiscores.h"
#include "cHUD.h"
#include "cGUI.h"
#include "cGameState.h"


//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// CLASS cAVRTContext
//
// a simple wrapper holding all 'global' runtime data in a nice namespace
// this contains all data and systems available when the game is running

class cAVRTContext
{
public:
	//-------------------------------------------------------------
	// Public data

	HINSTANCE				mAppInst;				// saved app instance

	c3DGameView				*m3DGameView;			// instance of renderer
	cGameState				*mGameState;			// current game logic

	cFrameLimiter			*mFrameLimiter;			// frame control

	//-------------------------------------------------------------
	// Public functions

	// constructor
	cAVRTContext (void);

};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

// single global instance of our data wrapper class
extern cAVRTContext		gAVRTContext;

// players name, defined on startup
extern sChar8 gPlayerName[64];


//------------------------------------------------------------------------------
// Global function declarations

////////////////////////////////////////////////////////////////////////////////
// PUBLIC assertWithMsg
//
// check a conditional, stop program execution and display an error message if failed

void assertWithMsg(
	sBool cond,
	sChar8 *fmt, 
	...);


#endif // HDD_ANTIVIRUS_CORE_ANTIVIRUS_H
