////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	avBasetypes.h
// Desc:	Some common type definitions and useful macros
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_AVBASETYPES_H
#define HDD_ANTIVIRUS_CORE_AVBASETYPES_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

#ifndef TRUE
#define TRUE		(1)
#endif

#ifndef FALSE
#define FALSE		(0)
#endif

#ifndef NULL
#define NULL		(0)
#endif

#ifndef INVALID
#define INVALID		(-1)
#endif

// size of an array
#define ARRAY_SIZE(n)	(sizeof(n)/sizeof((n)[0]))

// safe delete & release 
#define SAFE_FREE(p)		{ if(p) { free (p);			(p)=NULL; } }
#define SAFE_DELETE(p)		{ if(p) { delete (p);		(p)=NULL; } }
#define SAFE_RELEASE(p)		{ if(p) { (p)->Release();	(p)=NULL; } }

// macros to clamp values between ranges
#define CLAMP(var,low,high)			\
{									\
	if(var<low)						\
		var=low;					\
	else if(var>high)				\
		var=high;					\
}

#define CLAMP_LOW(var,low)			\
{									\
	if(var<low)						\
		var=low;					\
}

#define CLAMP_HIGH(var,high)		\
{									\
	if(var>high)					\
		var=high;					\
}


//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

// character types
typedef char				sChar;
typedef unsigned char		sUChar;
typedef unsigned short		sWChar;

// numbered character types
typedef char				sChar8;
typedef unsigned char		sUChar8;
typedef unsigned short		sUChar16;


// number types
typedef char				sInt8;
typedef unsigned char		sUInt8;
typedef unsigned short		sUInt16;
typedef short				sInt16;
typedef unsigned int		sUInt32;
typedef int					sInt32;
typedef unsigned __int64	sUInt64;
typedef __int64				sInt64;
typedef unsigned long		sULong;
typedef long				sLong;

// word-size types
typedef sUInt16				sWord;
typedef sUInt32				sDWord;

// floating point types
typedef float				sFloat;
typedef sFloat				sFloat32;

// boolean types
typedef sUChar				sByte;
typedef sInt32				sBool;


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_AVBASETYPES_H
