////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	avMain.h
// Desc:	Entrypoint for the game (contains WinMain)
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"


//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

// players name, defined on startup
sChar8 gPlayerName[64];


//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// CALLBACK startProc
//
// default window proc for our opening dialog

sBool CALLBACK startProc( 
	HWND hWnd,				// owner window
	UINT msg,				// incoming message code
	WPARAM wParam,			// generic input parameter 1
	LPARAM lParam)			// generic input parameter 2
{
	switch (msg)
	{
		case WM_INITDIALOG:
		{
			SetWindowText(GetDlgItem(hWnd, IDC_MYNAME), "Harry D");
		}
		break;

		case WM_COMMAND:
			{
				switch (LOWORD(wParam))
				{
					case IDOK:
						GetWindowText(GetDlgItem(hWnd, IDC_MYNAME), gPlayerName, 64);
						EndDialog(hWnd, 0);
						break;

					case IDCANCEL:
						exit(-1);
						break;
				}
				break;
			}
			break;

		default:
			return FALSE;
			break;
	}
	return TRUE;
}


////////////////////////////////////////////////////////////////////////////////
// PUBLIC gameStateRenderFn
//
// callback used to pass to 3D game view and render from gamestate class

void gameStateRenderFn(c3DGameView *gview)
{
	// render scene
	gAVRTContext.mGameState->draw(gview);
}

////////////////////////////////////////////////////////////////////////////////
// PUBLIC WinMain
//
// windows application entry point

sInt32 APIENTRY WinMain(
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
	LPTSTR    lpCmdLine,
	sInt32    nCmdShow)
{
    MSG msg;

	// spin the wheel of chance
	srand( (sUInt32)time( NULL ) );

	// start up RT context class
	gAVRTContext.mAppInst = hInstance;

	// show startup window
	DialogBox(
		hInstance,
		MAKEINTRESOURCE(IDD_STARTUP),
		NULL,
		startProc);

	gAVRTContext.m3DGameView = new c3DGameView();
	assertWithMsg( (gAVRTContext.m3DGameView != NULL), "c3DGameView could not be created in WinMain");
	gAVRTContext.m3DGameView->init(hInstance);

	gAVRTContext.mGameState = new cGameState();
	assertWithMsg( (gAVRTContext.mGameState != NULL), "mGameState could not be created in WinMain");
	gAVRTContext.mGameState->init();

	gAVRTContext.mFrameLimiter = new cFrameLimiter();
	assertWithMsg( (gAVRTContext.mFrameLimiter != NULL), "mFrameLimiter could not be created in WinMain");
	gAVRTContext.mFrameLimiter->init();

	// message loop
    PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE );
    while (msg.message != WM_QUIT)
    {
		gAVRTContext.mFrameLimiter->start();

		// update controller states, etc
		gAVRTContext.m3DGameView->update();

		// update game logic
		gAVRTContext.mGameState->update();

		// draw using gamestate class
		gAVRTContext.m3DGameView->render(gameStateRenderFn);

		// poll and clear the message queue if there
		// are any items available
        if (PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ))
        {
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }

		gAVRTContext.mFrameLimiter->stop();
	}

	delete gAVRTContext.mFrameLimiter;

	gAVRTContext.mGameState->tidyup();
	delete gAVRTContext.mGameState;

	gAVRTContext.m3DGameView->tidyup();
	delete gAVRTContext.m3DGameView;

	return 0;
}