////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	c2DRender.h
// Desc:	Bits and pieces required to support 2D-style rendering
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// STATIC METHOD applyViewMatrix
//
// apply a 2D projection matrix to the given view

void c2DRender::applyViewMatrix(
	c3DGameView *gview)
{
    D3DXMATRIX mtxOrtho, ident, mtxViewOff;
    
    //Setup the orthogonal projection matrix and the default world/view matrix
    D3DXMatrixOrthoLH(
		&mtxOrtho, 
		(sFloat32)gview->getWidth(), 
		(sFloat32)gview->getHeight(), 
		0.0f, 
		4.0f);
    D3DXMatrixIdentity(&ident);
	D3DXMatrixTranslation(
		&mtxViewOff, 
		-(sFloat32)(gview->getWidth() / 2), 
		(sFloat32)(gview->getHeight() / 2), 
		0.0f);

	// setup view matrices
    gview->getDevice()->SetTransform(D3DTS_PROJECTION, &mtxOrtho);
    gview->getDevice()->SetTransform(D3DTS_WORLD, &ident);
    gview->getDevice()->SetTransform(D3DTS_VIEW, &mtxViewOff);

    // set common render states used for 2D rendering
    gview->getDevice()->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);
    gview->getDevice()->SetRenderState(D3DRS_LIGHTING, FALSE);
}


////////////////////////////////////////////////////////////////////////////////
// METHOD c2DRender
//
// constructor : nullify any class memory, reset to defaults

c2DRender::c2DRender(void)
{
}

