////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	c3DRender.h
// Desc:	A place to store the setup for the main 3D rendered scene
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void c3DRender::init(void)
{
	D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );
	D3DLIGHT8 light0;

	// setup common lighting params
	memset(&light0, 0, sizeof(D3DLIGHT8));
	light0.Type = D3DLIGHT_DIRECTIONAL;
	light0.Diffuse = D3DXCOLOR(0.7f, 0.5f, 0.5f, 0.0f);
	light0.Attenuation0 = 1.0f;
	light0.Direction = D3DXVECTOR3(-0.25f, 0.25, 0.5f);
	light0.Range = 2048.0f;
	light0.Falloff = 1.0f;

	// default right-hand light
	gAVRTContext.m3DGameView->getDevice()->LightEnable(0, TRUE);
	gAVRTContext.m3DGameView->getDevice()->SetLight(
		0,
		&light0);
	
	// default left-hand light
    light0.Diffuse = D3DXCOLOR(0.5f, 0.5f, 0.7f, 0.0f);
	light0.Direction = D3DXVECTOR3(0.25f, -0.25, 0.5f);
	gAVRTContext.m3DGameView->getDevice()->LightEnable(1, TRUE);
	gAVRTContext.m3DGameView->getDevice()->SetLight(
		1,
		&light0);


	// build projection matrix
    D3DXMatrixPerspectiveFovLH( 
		&mProjMtx, 
		D3DXToRadian(24.0f), 
		gAVRTContext.m3DGameView->getAspect(), 
		0.5f,
		50.0f );

	// reset view matrix
	D3DXMatrixIdentity(&mViewMtx);

	// build a look-at view matrix
	D3DXVECTOR3 from(8.0f, -8.0f, -30.0f);
	D3DXVECTOR3 to(8.0f, -8.0f, 0.0f);
    D3DXMatrixLookAtLH( 
		&mViewMtx, 
		(D3DXVECTOR3*)&from, 
		(D3DXVECTOR3*)&to, 
		&vUpVec );
}

////////////////////////////////////////////////////////////////////////////////
// STATIC METHOD applyViewMatrix
//
// apply a 3D projection matrix to the given view

void c3DRender::applyViewMatrix(
	c3DGameView *gview)
{
	// set the projection matrix into the viewport device
    gAVRTContext.m3DGameView->getDevice()->SetTransform( D3DTS_PROJECTION, &mProjMtx );

    // set the app view matrix for normal viewing
	gAVRTContext.m3DGameView->getDevice()->SetTransform( D3DTS_VIEW, &mViewMtx );

	// default render states for main scene rendering
    gAVRTContext.m3DGameView->getDevice()->SetRenderState( D3DRS_LIGHTING, TRUE );
    gAVRTContext.m3DGameView->getDevice()->SetRenderState( D3DRS_ZENABLE, TRUE );
    gAVRTContext.m3DGameView->getDevice()->SetRenderState( D3DRS_LIGHTING, TRUE );
	gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_DISABLE );
}

////////////////////////////////////////////////////////////////////////////////
// METHOD c3DRender
//
// constructor : nullify any class memory, reset to defaults

c3DRender::c3DRender(void)
{
	memset(this, 0, sizeof(c3DRender));
}

