////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cCureChain.h
// Desc:	Similar to the virus chain, this supplies the user-laid cure blocks out
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CCURECHAIN_H
#define HDD_ANTIVIRUS_CORE_CCURECHAIN_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// CLASS cCureChain
//
// Controls a single cure chain present on the screen

class cCureChain
{
public:
	//-------------------------------------------------------------
	// Public data

	D3DXMATRIX			mTM;						// current position
	sInt32				mLateralIdx;				// lateral index to draw in X
	sFloat32			mYPos;						// Y position

	// the viral codes must match an incoming viral strand to destroy it
	sInt32				mNumViralCodes;				// number of codes for this cure block
	eViralCode			mViralCodes[gMaxNumViralCodeList];				// list of viral codes

	sBool				mActive;
	sFloat32			mYStart, mYEnd;				// cached start/end y positions init'd at ::create time

	sFloat32			mLife;						// time until fade out

	sBool				mIsDying;
	sFloat32			mDyingEmissive;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD create
	//
	// create a new sequence from the active cure sequence. this is called
	// by the gameplay manager

	void create(
		sInt32 lat,
		sFloat32 ypos,
		cCureSeq *cs);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD update
	//
	// called via game loop, once per frame. updates class for the current frame.

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD draw
	//
	// renders any graphics associated with this class (HUD, debug, etc)

	void draw(
		c3DGameView *gview);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD hasCollided
	//
	// return TRUE if this chain collides with the item defined by the inputs

	sBool hasCollided(
		sInt32 lat,
		sInt32 size,
		sFloat32 ypos,
		sInt32 &colIdx);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD viralMatch
	//
	// does this cure chain link with the given viral chain?

	sBool viralMatch(
		cViralChain *vc);


	// constructor
	cCureChain (void);

};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CCURECHAIN_H
