////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cCureData.h
// Desc:	Definitions for all cure data, colour and meshes, etc
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD renderCureBlock
//
// renders a cure block, does not set world TM

void cCureData::renderCureBlock(
	sFloat32 valpha,		// alpha value for this component
	eViralCode	vcode,		// one of the registered viral codes
	cCureChain *cc)			// chain data
{
	D3DMATERIAL8 rendMtl = gViralMaterials[(sInt32)vcode][0];
	
	if (cc)
	{
		if (cc->mIsDying)
		{
			rendMtl.Emissive = D3DXCOLOR(cc->mDyingEmissive, cc->mDyingEmissive, cc->mDyingEmissive, 1.0f);
		}
	}

	// setup and activate mtl
	rendMtl.Diffuse.a = valpha;
	gAVRTContext.m3DGameView->getDevice()->SetMaterial(&rendMtl);

	// draw mesh (theres only one mtl, so don't recurse!)
	mMesh->DrawSubset(0);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cCureData::init(void)
{
	DWORD numMtls;
	sChar8 loadPath[MAX_PATH];

	// form path to loadable data
	sprintf(loadPath, "data\\cure_seg.X");

	// check for existence of data
	assertWithMsg(PathFileExists(loadPath), "Could not load mesh data : %s", loadPath);

	// load X file (scully!)
	D3DXLoadMeshFromX(
		loadPath,
		D3DXMESH_MANAGED,
		gAVRTContext.m3DGameView->getDevice(),
		NULL,
		&mMtlBuffer,
		&numMtls,
		&mMesh);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cCureData
//
// constructor : nullify any class memory, reset to defaults

cCureData::cCureData(void)
{
	memset(this, 0, sizeof(cCureData));
}

