////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cCureSeq.h
// Desc:	The helper object showing current the cure sequence
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cCureSeq::init(void)
{
	mLatDiv = (gAVRTContext.m3DGameView->getWidth() / (16 + 1));
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cCureSeq::update(
	cGameplay *gplay)
{
	sInt32 latPos = 0, i;
	sFloat32 yPos;
	eViralCode newCode = (eViralCode)(NUM_VIRAL_CODES+1);
	eGameDifficulty gDiff = gAVRTContext.mGameState->getGameDiff();

	// check for a new code in the cure sequence
	if (mNumViralCodes < gMaxNumViralCodeList)
	{
		// A = RED
		if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_A))
		{
			newCode = VC_RED;
		}
		// S = GREEN
		if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_S))
		{
			newCode = VC_GREEN;
		}
		// D = BLUE
		if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_D))
		{
			newCode = VC_BLUE;
		}
		// F = YELLOW
		if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_F))
		{
			newCode = VC_YELLOW;
		}
		// G = PURPLE
		if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_G))
		{
			newCode = VC_PURPLE;
		}
		// H = WHITE
		if (gAVRTContext.m3DGameView->isKbdButtonJustPressed(DIK_H))
		{
			newCode = VC_WHITE;
		}

		// check code against difficulty level, add if possible
		if (newCode < gMaxViralCodeChoices[gDiff])
		{
			mViralCodes[mNumViralCodes++] = newCode;
		}
	}

	// RIGHT MOUSE cancels current sequence
	if (gAVRTContext.m3DGameView->isMouseButtonJustPressed(1))
	{
		mNumViralCodes = 0;
	}

	// MIDDLE MOUSE cancels last added item
	if (gAVRTContext.m3DGameView->isMouseButtonJustPressed(2))
	{
		mNumViralCodes --;
		CLAMP_LOW(mNumViralCodes, 0);
	}

	// work out laterial position from mouse pointer
	latPos = (gAVRTContext.mGameState->mMousePtr->mX / mLatDiv);
	CLAMP(latPos, 0, 16);

	// fudge a Y position from the mouse position, its not pretty, yet it works (ahem)
	yPos = 1.3f + ((sFloat32)gAVRTContext.mGameState->mMousePtr->mY / ((sFloat32)gAVRTContext.m3DGameView->getHeight() / 13.25f));

	CLAMP_HIGH(yPos, 12);

	// form tm matrix for cure sequence
	D3DXMatrixTranslation(&mTM, (sFloat32)latPos, -yPos, 0.0f);

	// LEFT MOUSE places the current sequence
	if (gAVRTContext.m3DGameView->isMouseButtonJustPressed(0))
	{
		for (i=0; i<gMaxActiveViralChains; i++)
		{
			if (!gplay->mCureChain[i].mActive)
			{
				gplay->mCureChain[i].create(latPos, -yPos, this);
				break;
			}
		}

		// reset sequence now
		mNumViralCodes = 0;
	}	
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cCureSeq::draw(
	c3DGameView *gview)
{
	sInt32 i;

	// draw the chain
	for (i=0; i<mNumViralCodes; i++)
	{
		// render mesh
		gview->getDevice()->SetTransform(D3DTS_WORLD, &mTM);
		gAVRTContext.mGameState->mCureData->renderCureBlock(
			0.75f,
			mViralCodes[i],
			NULL);

		// next slot in X
		mTM.m[3][0] += 1.0f;
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cCureSeq
//
// constructor : nullify any class memory, reset to defaults

cCureSeq::cCureSeq(void)
{
	memset(this, 0, sizeof(cCureSeq));
}

