////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cFrameLimiter.h
// Desc:	A looped manager to control framerate
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD readTime
//
// get current timing value

sFloat32 cFrameLimiter::readTime(void)
{
	LARGE_INTEGER time;
	if (!QueryPerformanceCounter(&time))
	{
		return (sFloat32)GetTickCount();
	}

	sFloat32 ftime = (sFloat32)(time.QuadPart);
	return ftime;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cFrameLimiter::init(void)
{
	LARGE_INTEGER freq;

	// poll the timer for its frequency in Hz
	if (QueryPerformanceFrequency(&freq))
	{
		mTicksPerFrame = freq.QuadPart;
	}
	else
	{
		mTicksPerFrame = 1000.0f;
	}

	// sort out the time we expect a frame to take at gFramerateF
	mRequiredFrameTime = (mTicksPerFrame / gFramerateF);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD start
//
// start at beginning of frame

void cFrameLimiter::start(void)
{
	// store current time counter
	mCurFrameTime = readTime();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD stop
//
// call at end of frame to lock framerate

void cFrameLimiter::stop(void)
{
	mNumFrames ++;

	// how long this frame took
	sFloat32 frameTime = readTime() - mCurFrameTime;
	sFloat32 timePro = (mRequiredFrameTime-frameTime);

	if (timePro > 0)
	{
		Sleep((sInt32)((timePro/mTicksPerFrame)*1000.0f));
	}

	// synch up to frame rate
//	while (frameTime < mRequiredFrameTime)
//	{
//		frameTime = readTime() - mCurFrameTime;
//	}	
}


////////////////////////////////////////////////////////////////////////////////
// METHOD cFrameLimiter
//
// constructor : nullify any class memory, reset to defaults

cFrameLimiter::cFrameLimiter(void)
{
	memset(this, 0, sizeof(cFrameLimiter));
}

