////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cFrameLimiter.h
// Desc:	A looped manager to control framerate
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CFRAMELIMITER_H
#define HDD_ANTIVIRUS_CORE_CFRAMELIMITER_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// CLASS cFrameLimiter
//
// Use to lock to a chosen framerate

class cFrameLimiter
{
protected:
	//-------------------------------------------------------------
	// Protected data

	//-------------------------------------------------------------
	// Protected functions


public:
	//-------------------------------------------------------------
	// Public data

	sInt32			mNumFrames;
	sFloat32		mCurFrameTime, 
					mRequiredFrameTime,
					mTicksPerFrame;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD readTime
	//
	// get current timing value

	sFloat32 readTime(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD start
	//
	// start at beginning of frame

	void start(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD stop
	//
	// call at end of frame to lock framerate

	void stop(void);


	// constructor
	cFrameLimiter (void);

};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

// the desired locked framerate
const sFloat32	gFramerateF		= 50.0f;

const sFloat32	gFrameTime		= 1.0f / gFramerateF;


//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CFRAMELIMITER_H
