////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cGUI.h
// Desc:	A simple GUI for changing options and starting the game
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cGUI::init(void)
{
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cGUI::update(void)
{
	sInt32 i, mouseY, drawY;

	// get mouse Y
	mouseY = gAVRTContext.mGameState->mMousePtr->mY;
	
	drawY = (sInt32)(0.3f * (sFloat32)gAVRTContext.m3DGameView->getHeight());
	if (gAVRTContext.m3DGameView->isMouseButtonJustPressed(0))
	{
		for (i=0; i<NUM_GAME_DIFF_SETTINGS; i++)
		{
			if (mouseY > drawY && mouseY < (drawY + (gFontHeight*2)))
			{
				// set difficulty
				gAVRTContext.mGameState->setGameDiff((eGameDifficulty)i);

				// get playing
				gAVRTContext.mGameState->setGameMode(GM_AV_PLAY);
			}

			drawY += (gFontHeight * 3);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cGUI::draw(
	c3DGameView *gview)
{
	RECT fontRect;
	sInt32 i, mouseY, alphaVar;

	// default position
	fontRect.right = gview->getWidth();
	fontRect.bottom = gview->getHeight();

	// switch to 2D mode
	c2DRender::applyViewMatrix(gview);

	// begin rendering
	gview->getFont()->Begin();

	// draw title
	fontRect.left = 0;
	fontRect.top = (sInt32)(0.05f * (sFloat32)gview->getHeight());
	gview->getFont()->DrawText(
		"- Choose Your Difficulty -",
		-1,
		&fontRect,
		DT_CENTER,
		D3DCOLOR_RGBA(255,192,0,255));

	fontRect.top = (sInt32)(0.9f * (sFloat32)gview->getHeight());
	gview->getFont()->DrawText(
		"Click A Choice To Play!",
		-1,
		&fontRect,
		DT_CENTER,
		D3DCOLOR_RGBA(255,192,0,255));

	// get mouse Y
	mouseY = gAVRTContext.mGameState->mMousePtr->mY;

	// draw difficulty settings
	fontRect.left = (sInt32)(0.05f * (sFloat32)gview->getWidth());
	fontRect.top = (sInt32)(0.3f * (sFloat32)gview->getHeight());
	for (i=0; i<NUM_GAME_DIFF_SETTINGS; i++)
	{
		// draw diff name
		gview->getFont()->DrawText(
			gDifficultyName[i],
			-1,
			&fontRect,
			DT_LEFT,
			D3DCOLOR_RGBA(255,255,255,255));

		// highlight one under mouse
		if (mouseY > fontRect.top && mouseY < (fontRect.top + (gFontHeight*3)))
		{
			alphaVar = 255;
		}
		else
		{
			alphaVar = 128;
		}

		// draw description
		fontRect.top += gFontHeight;
		gview->getFont()->DrawText(
			gDifficultyDesc[i],
			-1,
			&fontRect,
			DT_LEFT,
			D3DCOLOR_RGBA(255,255,255,alphaVar));

		fontRect.top += (gFontHeight * 2);
	}

	// finish rendering
	gview->getFont()->End();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cGUI::tidyup(void)
{
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cGUI
//
// constructor : nullify any class memory, reset to defaults

cGUI::cGUI(void)
{
}

