////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cGameState.h
// Desc:	Controls the flow of game logic
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CGAMESTATE_H
#define HDD_ANTIVIRUS_CORE_CGAMESTATE_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

////////////////////////////////////////////////////////////////////////////////
// ENUM eGameMode
//
// the master list of available game modes that can be running in antivirus

typedef enum eGameMode
{
	GM_AV_GUI,
	GM_AV_PLAY,
	GM_AV_GAMEOVER,
	GM_AV_HISCORES,

	NUM_GAME_MODES

} eGameMode;


//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// CLASS cGameState
//
// A management class controlling what the user gets to play with and how
// the game progresses. basically a big ol state machine.

class cGameState
{
private:
	//-------------------------------------------------------------
	// Private data

	//-------------------------------------------------------------
	// Private functions


protected:
	//-------------------------------------------------------------
	// Protected data

	eGameMode				mGameMode;			// active game mode
	eGameDifficulty			mGameDiff;			// active game difficulty


	//-------------------------------------------------------------
	// Protected functions


public:
	//-------------------------------------------------------------
	// Public data

	// viral & cure data management classes
	cViralData				*mViralData;
	cCureData				*mCureData;

	// a mouse pointer used throughout the game
	cMousePtr				*mMousePtr;

	// simply, a hiscore table
	cHiscores				*mHiscores;

	// wow, its a gui manager
	cGUI					*mGUI;

	// the main game code
	cGameplay				*mGameplay;
	cHUD					*mHUD;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD update
	//
	// called via game loop, once per frame. updates class for the current frame.

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD draw
	//
	// renders any graphics associated with this class (HUD, debug, etc)

	void draw(
		c3DGameView *gview);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD tidyup
	//
	// releases any memory, returns to defaults, shutsdown the class after use

	void tidyup(void);


	// constructor
	cGameState (void);


	// Accessors
	eGameMode		getGameMode(void) { return mGameMode; }
	void			setGameMode(eGameMode gm);
	eGameDifficulty	getGameDiff(void) { return mGameDiff; }
	void			setGameDiff(eGameDifficulty gd) { mGameDiff = gd; }
};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

// text descriptions for difficulty modes
extern sChar8	*gDifficultyName[NUM_GAME_DIFF_SETTINGS];
extern sChar8	*gDifficultyDesc[NUM_GAME_DIFF_SETTINGS];


//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CGAMESTATE_H
