////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cGameplay.h
// Desc:	Code and managers running the main show - the antivirus game code
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CGAMEPLAY_H
#define HDD_ANTIVIRUS_CORE_CGAMEPLAY_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

const sInt32 gMaxActiveViralChains	=	20;


//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// CLASS cGameplay
//
// The core class that manages the flow of the game

class cGameplay
{
friend cCureSeq;
private:
	//-------------------------------------------------------------
	// Private data

	//-------------------------------------------------------------
	// Private functions


protected:
	//-------------------------------------------------------------
	// Protected data

	sUInt32			mCurScore;			// players current score
	sInt32			mHealth;			// health value, 0 - <max>
	sInt32			mMaxHealth;			// maximum health value for this diff level

	// list of all running chains
	cViralChain		mViralChain[gMaxActiveViralChains];
	sInt32			mLastSpawnIdx;

	sFloat32		mTimeTillNextSpawn;

	// cure chains
	cCureChain		mCureChain[gMaxActiveViralChains];
	cCureSeq		*mCureSeq;


	//-------------------------------------------------------------
	// Protected functions


public:
	//-------------------------------------------------------------
	// Public data

	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD reset
	//
	// reset the class back to default state

	void reset(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD update
	//
	// called via game loop, once per frame. updates class for the current frame.

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD draw
	//
	// renders any graphics associated with this class (HUD, debug, etc)

	void draw(
		c3DGameView *gview);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD tidyup
	//
	// releases any memory, returns to defaults, shutsdown the class after use

	void tidyup(void);


	// constructor
	cGameplay (void);


	// Accessors
	sUInt32		getScore(void) { return mCurScore; }
	sInt32		getHealth(void) { return mHealth; }
	sInt32		getMaxHealth(void) { return mMaxHealth; }
};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CGAMEPLAY_H
