////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cHUD.h
// Desc:	A drawing system for the gameplay HUD overlay
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cHUD::init(void)
{
	sHUDVtx *vertexBuffer = NULL;
	sFloat32 scnW = (sFloat32)gAVRTContext.m3DGameView->getWidth();

	reset();

	// load the skin texture
	D3DXCreateTextureFromFile(
		gAVRTContext.m3DGameView->getDevice(),
		"data\\skin.tga",
		&mSkinTex);

	// load the blood texture
	D3DXCreateTextureFromFile(
		gAVRTContext.m3DGameView->getDevice(),
		"data\\blood.tga",
		&mBloodTex);

	// create a vertex buffer to hold all vertices
    gAVRTContext.m3DGameView->getDevice()->CreateVertexBuffer( 
		4 * sizeof(sHUDVtx),
		0,
		D3DFVF_HUD_VERTEX,
		D3DPOOL_DEFAULT,
		&mVertexBufferSkin );

	// lock it to get access to the data
    mVertexBufferSkin->Lock( 0, 0, (BYTE**)&vertexBuffer, 0 );

	vertexBuffer[0].mPos = D3DXVECTOR3(0, 0.0f, 2.0f);
	vertexBuffer[1].mPos = D3DXVECTOR3(0, 32.0f, 2.0f);
	vertexBuffer[2].mPos = D3DXVECTOR3(scnW, 0.0f, 2.0f);
	vertexBuffer[3].mPos = D3DXVECTOR3(scnW, 32.0f, 2.0f);

	vertexBuffer[0].mU = 0.0f;
    vertexBuffer[0].mV = 1.0f;
    vertexBuffer[1].mU = 0.0f;
    vertexBuffer[1].mV = 0.0f;
    vertexBuffer[2].mU = 12.0f;
    vertexBuffer[2].mV = 1.0f;
    vertexBuffer[3].mU = 12.0f;
    vertexBuffer[3].mV = 0.0f;

	// release lock on vertex buffer
    mVertexBufferSkin->Unlock();


	// create a vertex buffer to hold all vertices for blood stream
    gAVRTContext.m3DGameView->getDevice()->CreateVertexBuffer( 
		4 * sizeof(sHUDVtx),
		0,
		D3DFVF_HUD_VERTEX,
		D3DPOOL_DEFAULT,
		&mVertexBufferBlood );

	// lock it to get access to the data
    mVertexBufferBlood->Lock( 0, 0, (BYTE**)&vertexBuffer, 0 );

	vertexBuffer[0].mPos = D3DXVECTOR3(0, 0.0f, 2.0f);
	vertexBuffer[1].mPos = D3DXVECTOR3(0, 64.0f, 2.0f);
	vertexBuffer[2].mPos = D3DXVECTOR3(scnW, 0.0f, 2.0f);
	vertexBuffer[3].mPos = D3DXVECTOR3(scnW, 64.0f, 2.0f);

	vertexBuffer[0].mCol = 
		vertexBuffer[1].mCol = 
		vertexBuffer[2].mCol =
		vertexBuffer[3].mCol = D3DCOLOR_RGBA(128,128,128,255);

	vertexBuffer[0].mU = 0.0f;
    vertexBuffer[0].mV = 1.0f;
    vertexBuffer[1].mU = 0.0f;
    vertexBuffer[1].mV = 0.0f;
    vertexBuffer[2].mU = 12.0f;
    vertexBuffer[2].mV = 1.0f;
    vertexBuffer[3].mU = 12.0f;
    vertexBuffer[3].mV = 0.0f;

	// release lock on vertex buffer
    mVertexBufferBlood->Unlock();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD reset
//
// reset the class back to default state

void cHUD::reset(void)
{
	mBloodX = mBloodY = 0.0f;
	mBloodPulse = 0;
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cHUD::update(void)
{
	sHUDVtx *vertexBuffer = NULL;
	sFloat32 vSin = sinf(mBloodY) * 0.5f;
	sFloat32 xSin = sinf(mBloodY);

	// update blood UV positions
	mBloodY += 0.015f;
	mBloodX += 0.025f;
	if (mBloodX >= 1.0f)
	{
		mBloodX = 0.0f;
	}

	// update blood green pulsing on infection
	mBloodPulse -= 4;
	CLAMP_LOW(mBloodPulse, 0);

	// lock it to get access to the data
    mVertexBufferBlood->Lock( 0, 0, (BYTE**)&vertexBuffer, 0 );

	vertexBuffer[0].mU = mBloodX + xSin;
    vertexBuffer[0].mV = vSin + 1.0f;
    vertexBuffer[1].mU = mBloodX + 0.0f;
    vertexBuffer[1].mV = vSin;
    vertexBuffer[2].mU = mBloodX + 12.0f;
    vertexBuffer[2].mV = vSin + 1.0f;
    vertexBuffer[3].mU = mBloodX + xSin + 12.0f;
    vertexBuffer[3].mV = vSin;

	vertexBuffer[0].mCol = 
		vertexBuffer[2].mCol = D3DCOLOR_RGBA(128-mBloodPulse,128+mBloodPulse,128-mBloodPulse,255);

	// release lock on vertex buffer
    mVertexBufferBlood->Unlock();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cHUD::draw(
	c3DGameView *gview)
{
	RECT fontRect;
	D3DXMATRIX tempTM;
	sChar8 txtTmp[64];
	sFloat32 hlPct;
	sFloat32 scnH = (sFloat32)gAVRTContext.m3DGameView->getHeight();


	// switch to 2D mode
	c2DRender::applyViewMatrix(gview);

	// score render position
	fontRect.left = 0;
	fontRect.top = 8;
	fontRect.right = gview->getWidth() - 8;
	fontRect.bottom = gview->getHeight();

	hlPct = 100.0f / (sFloat32)gAVRTContext.mGameState->mGameplay->getMaxHealth();

	sprintf(txtTmp, "Score : %08i\nHealth : %03i%%", 
		gAVRTContext.mGameState->mGameplay->getScore(),
		(sInt32)((sFloat32)gAVRTContext.mGameState->mGameplay->getHealth() * hlPct) );

	// begin rendering
	gview->getFont()->Begin();

	// draw current score
	gview->getFont()->DrawText(
		txtTmp,
		-1,
		&fontRect,
		DT_RIGHT,
		D3DCOLOR_RGBA(255,192,0,255));

	// finish rendering
	gview->getFont()->End();


    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1 );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );

	gAVRTContext.m3DGameView->getDevice()->SetVertexShader(D3DFVF_HUD_VERTEX);


	// draw skin layer
    D3DXMatrixTranslation(&tempTM, 0.0f, -(scnH - 64.0f), 0.0f);
    gAVRTContext.m3DGameView->getDevice()->SetTransform(D3DTS_WORLD, &tempTM);
    gAVRTContext.m3DGameView->getDevice()->SetTexture(0, mSkinTex);
	gAVRTContext.m3DGameView->getDevice()->SetStreamSource(0, mVertexBufferSkin, sizeof(sHUDVtx));
    gAVRTContext.m3DGameView->getDevice()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);

    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE2X );

	// draw blood flow
	D3DXMatrixTranslation(&tempTM, 0.0f, -scnH, 0.0f);
	gAVRTContext.m3DGameView->getDevice()->SetTransform(D3DTS_WORLD, &tempTM);
	gAVRTContext.m3DGameView->getDevice()->SetTexture(0, mBloodTex);
	gAVRTContext.m3DGameView->getDevice()->SetStreamSource(0, mVertexBufferBlood, sizeof(sHUDVtx));
	gAVRTContext.m3DGameView->getDevice()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cHUD::tidyup(void)
{
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cHUD
//
// constructor : nullify any class memory, reset to defaults

cHUD::cHUD(void)
{
	memset(this, 0, sizeof(cHUD));
}

