////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cHUD.h
// Desc:	A drawing system for the gameplay HUD overlay
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CHUD_H
#define HDD_ANTIVIRUS_CORE_CHUD_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

#define D3DFVF_HUD_VERTEX (D3DFVF_XYZ|D3DFVF_DIFFUSE|D3DFVF_TEX1)


//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// STRUCT sHUDVtx
//
// a simple vertex structure for HUD panels

typedef struct sHUDVtx
{
    D3DXVECTOR3		mPos;
	DWORD			mCol;
    sFloat32		mU, mV;

} sHUDVtx;


////////////////////////////////////////////////////////////////////////////////
// CLASS cHUD
//
// ** TODO: Description of class functionality ** 

class cHUD
{
private:
	//-------------------------------------------------------------
	// Private data

	//-------------------------------------------------------------
	// Private functions


protected:
	//-------------------------------------------------------------
	// Protected data

	// geometry storage buffers
	IDirect3DVertexBuffer8		*mVertexBufferSkin;
	IDirect3DVertexBuffer8		*mVertexBufferBlood;

	IDirect3DTexture8			*mSkinTex;
	IDirect3DTexture8			*mBloodTex;

	// blood UV movement variables
	sFloat32					mBloodX, mBloodY;


	//-------------------------------------------------------------
	// Protected functions


public:
	//-------------------------------------------------------------
	// Public data

	sInt32					mBloodPulse;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(void);
	
	////////////////////////////////////////////////////////////////////////////////
	// METHOD reset
	//
	// reset the class back to default state

	void reset(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD update
	//
	// called via game loop, once per frame. updates class for the current frame.

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD draw
	//
	// renders any graphics associated with this class (HUD, debug, etc)

	void draw(
		c3DGameView *gview);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD tidyup
	//
	// releases any memory, returns to defaults, shutsdown the class after use

	void tidyup(void);


	// constructor
	cHUD (void);

};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CHUD_H
