////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cHiscores.h
// Desc:	Every game needs a hiscore table!
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CHISCORES_H
#define HDD_ANTIVIRUS_CORE_CHISCORES_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

#define MAX_HISE_NAME			(64)
#define MAX_HIS_ENTRIES			(12)


//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

////////////////////////////////////////////////////////////////////////////////
// STRUCT sHiscoreEntry
//
// a single entry in the Hiscore table

typedef struct sHiscoreEntry
{
	sChar8				mName[MAX_HISE_NAME];
	sUInt32				mScore;
	eGameDifficulty		mDiffLevel;

} sHiscoreEntry;


////////////////////////////////////////////////////////////////////////////////
// CLASS cHiscores
//
// ** TODO: Description of class functionality ** 

class cHiscores
{
private:
	//-------------------------------------------------------------
	// Private data

	ID3DXFont				*mFontTitle;	// a larger font used to title the screen


	//-------------------------------------------------------------
	// Private functions


protected:
	//-------------------------------------------------------------
	// Protected data

	// the actual Hiscore table
	sHiscoreEntry			mHSTable[MAX_HIS_ENTRIES];

	sInt32					mAlphaFade;

	//-------------------------------------------------------------
	// Protected functions


public:
	//-------------------------------------------------------------
	// Public data

	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD reset
	//
	// reset the class back to default state

	void reset(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD genDefaults
	//
	// build default scores, always called before loading official ones

	void genDefaults(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD addScore
	//
	// adds the score with the current global player name to the chart

	void addScore(
		sUInt32 score);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD load
	//
	// load saved scores, if they are available

	void load(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD save
	//
	// save current scores

	void save(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD update
	//
	// called via game loop, once per frame. updates class for the current frame.

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD draw
	//
	// renders any graphics associated with this class (HUD, debug, etc)

	void draw(
		c3DGameView *gview);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD tidyup
	//
	// releases any memory, returns to defaults, shutsdown the class after use

	void tidyup(void);



	// constructor
	cHiscores (void);

};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CHISCORES_H
