////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cMousePtr.h
// Desc:	A mouse pointer graphic
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

// size of mouse ptr renderable panel
sFloat32 gMousePtrPanelSz		= 8.0f;


//-----------------------------------------------------------------------------
// Global data definitions

//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cMousePtr::init(void)
{
	sMousePtrVtx *vertexBuffer = NULL;

	// slap mouse into middle of view screen
	mX = gAVRTContext.m3DGameView->getWidth() / 2;
	mY = gAVRTContext.m3DGameView->getHeight() / 2;

	// load the pointer texture
	D3DXCreateTextureFromFile(
		gAVRTContext.m3DGameView->getDevice(),
		"data\\mouseptr.tga",
		&mMousePtrTex);

	// create the pointer geometry (just a flat panel)

	// create a vertex buffer to hold all vertices
    gAVRTContext.m3DGameView->getDevice()->CreateVertexBuffer( 
		4 * sizeof(sMousePtrVtx),
		0,
		D3DFVF_MOUSEPTR_VERTEX,
		D3DPOOL_DEFAULT,
		&mVertexBuffer );

	// lock it to get access to the data
    mVertexBuffer->Lock( 0, 0, (BYTE**)&vertexBuffer, 0 );

	vertexBuffer[0].mPos = D3DXVECTOR3(-gMousePtrPanelSz, -gMousePtrPanelSz, 0.0f);
	vertexBuffer[1].mPos = D3DXVECTOR3(-gMousePtrPanelSz, gMousePtrPanelSz, 0.0f);
	vertexBuffer[2].mPos = D3DXVECTOR3(gMousePtrPanelSz, -gMousePtrPanelSz, 0.0f);
	vertexBuffer[3].mPos = D3DXVECTOR3(gMousePtrPanelSz, gMousePtrPanelSz, 0.0f);

	vertexBuffer[0].mU = 0.0f;
    vertexBuffer[0].mV = 1.0f;
    vertexBuffer[1].mU = 0.0f;
    vertexBuffer[1].mV = 0.0f;
    vertexBuffer[2].mU = 1.0f;
    vertexBuffer[2].mV = 1.0f;
    vertexBuffer[3].mU = 1.0f;
    vertexBuffer[3].mV = 0.0f;

	// release lock on vertex buffer
    mVertexBuffer->Unlock();
}

////////////////////////////////////////////////////////////////////////////////
// METHOD update
//
// called via game loop, once per frame. updates class for the current frame.

void cMousePtr::update(void)
{
	sInt32 mXLimit = gAVRTContext.m3DGameView->getWidth(),
			mYLimit = gAVRTContext.m3DGameView->getHeight();

	// update screen position from DIO variables
	mX += gAVRTContext.m3DGameView->getGameDIO()->mCurFrameData.mMouse.lX;
	mY += gAVRTContext.m3DGameView->getGameDIO()->mCurFrameData.mMouse.lY;
	CLAMP(mX, 0, mXLimit);
	CLAMP(mY, 0, mYLimit);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD draw
//
// renders any graphics associated with this class (HUD, debug, etc)

void cMousePtr::draw(void)
{
	D3DXMATRIX mouseTM;

	// NOTE: a ortho 2D view matrix must alread be in effect
	// when this code kicks off - the other gamestate systems should leave in 2D

	// set transformation to draw mouse panel
    D3DXMatrixTranslation(&mouseTM, (sFloat32)mX, -(sFloat32)mY, 0.0f);
    gAVRTContext.m3DGameView->getDevice()->SetTransform(D3DTS_WORLD, &mouseTM);

	// set mouse pointer texture
    gAVRTContext.m3DGameView->getDevice()->SetTexture(0, mMousePtrTex);
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1 );
    gAVRTContext.m3DGameView->getDevice()->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );

	// set rendering source
	gAVRTContext.m3DGameView->getDevice()->SetStreamSource(0, mVertexBuffer, sizeof(sMousePtrVtx));
	gAVRTContext.m3DGameView->getDevice()->SetVertexShader(D3DFVF_MOUSEPTR_VERTEX);

	// draw the panel
    gAVRTContext.m3DGameView->getDevice()->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD tidyup
//
// releases any memory, returns to defaults, shutsdown the class after use

void cMousePtr::tidyup(void)
{
	SAFE_RELEASE(mMousePtrTex);
	SAFE_RELEASE(mVertexBuffer);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cMousePtr
//
// constructor : nullify any class memory, reset to defaults

cMousePtr::cMousePtr(void)
{
	memset(this, 0, sizeof(cMousePtr));
}

