////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cViralChain.h
// Desc:	Management for a single viral strand
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CVIRALCHAIN_H
#define HDD_ANTIVIRUS_CORE_CVIRALCHAIN_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

// maximum size of viral chain
const sInt32 gMaxNumViralCodeList = (8);

// distance at which a chain registers impact with the host (infection!)
const sFloat32 gMaxFallPosBeforeDet = (-13.0f);


//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

class cGameplay;

////////////////////////////////////////////////////////////////////////////////
// CLASS cViralChain
//
// A single viral chain

class cViralChain
{
public:
	//-------------------------------------------------------------
	// Public data

	D3DXMATRIX			mTM;						// current position
	sInt32				mLateralIdx;				// lateral index to draw in X
	sFloat32			mFallPos, mFallSpeed;		// vertical location, speed to fall per frame
	sFloat32			mCurYRot, mYRotSpeed;		// current spin and spin rate

	sInt32				mNumViralCodes;				// number of codes for this strand
	eViralCode			mViralCodes[gMaxNumViralCodeList];				// list of viral codes
	sInt32				mCurMeshVariant[gMaxNumViralCodeList];			// which mesh we're using

	sBool				mActive;

	sBool				mIsPowerup;

	sBool				mIsDying;
	sFloat32			mDyingEmissive;


	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD spawn
	//
	// spawn a new viral chain, at the given point with the correct parameters

	void spawn(
		sInt32 latIdx,				// x slot to spawn into
		sInt32 size,				// length of chain
		sBool powerup);				// is this a powerup

	////////////////////////////////////////////////////////////////////////////////
	// METHOD update
	//
	// called via game loop, once per frame. updates class for the current frame.

	void update(void);

	////////////////////////////////////////////////////////////////////////////////
	// METHOD draw
	//
	// renders any graphics associated with this class (HUD, debug, etc)

	void draw(
		c3DGameView *gview);


	// constructor
	cViralChain (void);

	// Accessors
	sBool hasInfected(void) 
	{ 
		if (!mActive || mIsDying) return FALSE;
		else return (mFallPos < gMaxFallPosBeforeDet); 
	}
};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

// the maximum choice of viral codes per difficultly level
extern sInt32 gMaxViralCodeChoices[NUM_GAME_DIFF_SETTINGS];


//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CVIRALCHAIN_H
