////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cViralData.h
// Desc:	Definitions for all viral data, colour and meshes, etc
////////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Include directives

#include "antivirus.h"

//-----------------------------------------------------------------------------
// Macro definitions

//-----------------------------------------------------------------------------
// Enumerations

//-----------------------------------------------------------------------------
// Type definitions

//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data definitions

// list of generated D3D materials for each possible viral state
D3DMATERIAL8 gViralMaterials[NUM_VIRAL_CODES][2];


//-----------------------------------------------------------------------------
// Static data definitions

//-----------------------------------------------------------------------------
// Forward declaration of functions

//-----------------------------------------------------------------------------
// Function definitions

////////////////////////////////////////////////////////////////////////////////
// METHOD renderChainLink
//
// renders a virus spiral mesh, does not set world TM

void cViralData::renderChainLink(
	sFloat32 valpha,		// alpha value for this component
	eViralCode vcode,		// one of the registered viral codes
	sInt32 meshID,			// meshID, < gNumViralMeshes
	cViralChain *vc)		// chain data
{
	D3DMATERIAL8 rendMtl = gViralMaterials[(sInt32)vcode][vc->mIsPowerup];

	if (vc->mIsDying)
	{
		rendMtl.Emissive = D3DXCOLOR(vc->mDyingEmissive, vc->mDyingEmissive, vc->mDyingEmissive, 1.0f);
	}

	// setup and activate mtl
	rendMtl.Diffuse.a = valpha;
	gAVRTContext.m3DGameView->getDevice()->SetMaterial(&rendMtl);

	// draw mesh (theres only one mtl, so don't recurse!)
	mVirusSpiral[meshID].mMesh->DrawSubset(0);
}

////////////////////////////////////////////////////////////////////////////////
// METHOD init
//
// initialise the class to defaults, ready for use

void cViralData::init(void)
{
	sInt32 i, j;
	DWORD numMtls;
	sChar8 loadPath[MAX_PATH];

	// load all viral spiral meshes from disk
	for (i=0; i<gNumViralMeshes; i++)
	{
		// form path to loadable data
		sprintf(loadPath, "data\\virus_seg%i.X", (i+1));

		// check for existence of data
		assertWithMsg(PathFileExists(loadPath), "Could not load mesh data : %s", loadPath);

		// load X file (scully!)
		D3DXLoadMeshFromX(
			loadPath,
			D3DXMESH_MANAGED,
			gAVRTContext.m3DGameView->getDevice(),
			NULL,
			&mVirusSpiral[i].mMtlBuffer,
			&numMtls,
			&mVirusSpiral[i].mMesh);
	}

	// setup D3D materials for viral codes
	for (i=0; i<NUM_VIRAL_CODES ;i++)
	{
		for (j=0; j<2; j++)
		{
			// reset mtl
			memset(&gViralMaterials[i][j], 0, sizeof(D3DMATERIAL8));

			// some basic emissive
			gViralMaterials[i][j].Emissive.r = 
				gViralMaterials[i][j].Emissive.g = 
				gViralMaterials[i][j].Emissive.b = 0.1f;

			// build a material set based upon the colour the viral code represents
			switch ((eViralCode)i)
			{
				case VC_RED:
					gViralMaterials[i][j].Diffuse = D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f);
					break;

				case VC_GREEN:
					gViralMaterials[i][j].Diffuse = D3DXCOLOR(0.0f, 1.0f, 0.0f, 1.0f);
					break;

				case VC_BLUE:
					gViralMaterials[i][j].Diffuse = D3DXCOLOR(0.0f, 0.0f, 1.0f, 1.0f);
					break;

				case VC_YELLOW:
					gViralMaterials[i][j].Diffuse = D3DXCOLOR(1.0f, 1.0f, 0.0f, 1.0f);
					break;

				case VC_PURPLE:
					gViralMaterials[i][j].Diffuse = D3DXCOLOR(1.0f, 0.0f, 1.0f, 1.0f);
					break;

				case VC_WHITE:
					gViralMaterials[i][j].Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);
					break;
			}

		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// METHOD cViralData
//
// constructor : nullify any class memory, reset to defaults

cViralData::cViralData(void)
{
	memset(this, 0, sizeof(cViralData));
}

