////////////////////////////////////////////////////////////////////////////////
// Copyright  2002 Harry Denholm
//
// Created for Ludum Dare 48Hr Coding Competition, July 2002 [www.ludumdare.com]
// Released free to the public, but use at your own risk!
//
// Family:	Antivirus Core
// File:	cViralData.h
// Desc:	Definitions for all viral data, colour and meshes, etc
////////////////////////////////////////////////////////////////////////////////

#ifndef HDD_ANTIVIRUS_CORE_CVIRALDATA_H
#define HDD_ANTIVIRUS_CORE_CVIRALDATA_H


//-----------------------------------------------------------------------------
// Include directives

//-----------------------------------------------------------------------------
// Macro definitions

// number of available virus spiral X files available [data\virus_seg<number>.X]
const sInt32 gNumViralMeshes		= 5;


//-----------------------------------------------------------------------------
// Enumerations

////////////////////////////////////////////////////////////////////////////////
// ENUM eViralCode
//
// a list of all available viral types, named in colour so that they correspond
// to their visual effect on screen. some of these are masked off at lower
// difficulty levels

typedef enum eViralCode
{
	VC_RED,
	VC_GREEN,
	VC_BLUE,
	VC_YELLOW,
	VC_PURPLE,
	VC_WHITE,

	NUM_VIRAL_CODES

} eViralCode;

//-----------------------------------------------------------------------------
// Type definitions

class cViralChain;

////////////////////////////////////////////////////////////////////////////////
// STRUCT cVirusSpiral
//
// the data required to render a single spiral, one loaded per number of spiral
// variations defined as gNumViralMeshes

typedef struct cVirusSpiral
{
	ID3DXBuffer		*mMtlBuffer;
	ID3DXMesh		*mMesh;

} cVirusSpiral;


////////////////////////////////////////////////////////////////////////////////
// CLASS cViralData
//
// Container class managing the data for drawing viral chains 

class cViralData
{
private:
	//-------------------------------------------------------------
	// Private data

	//-------------------------------------------------------------
	// Private functions


protected:
	//-------------------------------------------------------------
	// Protected data

	// list of virus meshes, loaded at startup
	cVirusSpiral		mVirusSpiral[gNumViralMeshes];


	//-------------------------------------------------------------
	// Protected functions


public:
	//-------------------------------------------------------------
	// Public data

	//-------------------------------------------------------------
	// Public functions

	////////////////////////////////////////////////////////////////////////////////
	// METHOD renderChainLink
	//
	// renders a virus spiral mesh, does not set world TM

	void renderChainLink(
		sFloat32 valpha,		// alpha value for this component
		eViralCode	vcode,		// one of the registered viral codes
		sInt32 meshID,			// meshID, < gNumViralMeshes
		cViralChain *vc);		// chain data

	////////////////////////////////////////////////////////////////////////////////
	// METHOD init
	//
	// initialise the class to defaults, ready for use

	void init(void);


	// constructor
	cViralData (void);

};


//-----------------------------------------------------------------------------
// Constants

//-----------------------------------------------------------------------------
// Global data declarations

// list of generated D3D materials for each possible viral state (+ powerup switch)
extern D3DMATERIAL8 gViralMaterials[NUM_VIRAL_CODES][2];


//------------------------------------------------------------------------------
// Global function declarations



#endif // HDD_ANTIVIRUS_CORE_CVIRALDATA_H
